package collections.comparators;


import java.util.Comparator;
import java.util.List;

/** 
 * A Comparator which imposes a specific order on a specific set of Objects.
 * Objects are presented to the FixedOrderComparator in a specified order and
 * subsequent calls to {@link #compare(Object, Object) compare} yield that order.
 * For example:
 * <pre>
 * String[] planets = {"Mercury", "Venus", "Earth", "Mars"};
 * FixedOrderComparator distanceFromSun = new FixedOrderComparator(planets);
 * Arrays.sort(planets);                     // Sort to alphabetical order
 * Arrays.sort(planets, distanceFromSun);    // Back to original order
 * </pre>
 * <p>
 * Once <code>compare</code> has been called, the FixedOrderComparator is locked
 * and attempts to modify it yield an UnsupportedOperationException.
 * <p>
 * 
 */
public class FixedOrderComparator implements Comparator {

    /** 
     * Behavior when comparing unknown Objects:
     * unknown objects compare as coming before known Objects.
     */
    public static final int UNKNOWN_BEFORE = 0;

    /** 
     * Behavior when comparing unknown Objects:
     * unknown objects compare as coming after known Objects.
     */
    public static final int UNKNOWN_AFTER = 1;

    /** 
     * Behavior when comparing unknown Objects:
     * unknown objects cause a IllegalArgumentException to be thrown.
     * This is the default behavior.
     */
    public static final int UNKNOWN_THROW_EXCEPTION = 2;

    //
    // Add further member variables as necessary
    // ...
    //

    // Constructors
    //-----------------------------------------------------------------------
    /** 
     * Constructs an empty FixedOrderComparator.
     */
    public FixedOrderComparator() {

    }

    /** 
     * Constructs a FixedOrderComparator which uses the order of the given array
     * to compare the objects.
     * <p>
     * The elements of the array are stored locally (see {@link #add(Object)}).
     * 
     * @param items  the array of items that the comparator can compare in order
     * @throws IllegalArgumentException if the array <code>items</code> is <code>null</code>
     */
    public FixedOrderComparator(Object[] items) {
        // TODO: complete
    }

    /** 
     * Constructs a FixedOrderComparator which uses the order of the given list
     * to compare the objects.
     * <p>
     * The elements of the list are stored locally (see {@link #add(Object)}).
     * 
     * @param items  the {@link List} of items that the comparator can compare in order
     * @throws IllegalArgumentException if the {@link List} is <code>null</code>.
     */
    public FixedOrderComparator(List items) {
        // TODO: complete
    }

    // Bean methods / state querying methods
    //-----------------------------------------------------------------------
    /**
     * Returns true if modifications cannot be made to the FixedOrderComparator.
     * FixedOrderComparators cannot be modified once they have performed a comparison.
     * 
     * @return true if modifications cannot be made to this comparator, false if it can be changed.
     */
    public boolean isLocked() {
        // TODO: complete
    	return false; // remove
    }

    /** 
     * Gets the behavior for comparing unknown objects, which is
     * UNKNOWN_THROW_EXCEPTION by default, and can be set
     * via {@link #setUnknownObjectBehavior(int)}.
     * 
     * @return the value for unknown behaviour - UNKNOWN_AFTER,
     * UNKNOWN_BEFORE or UNKNOWN_THROW_EXCEPTION
     */
    public int getUnknownObjectBehavior() {
        // TODO: complete
    	return -1; // remove
    }

    /** 
     * Sets the behavior for comparing unknown objects.
     * 
     * @param unknownObjectBehavior  the value for unknown behaviour -
     * UNKNOWN_AFTER, UNKNOWN_BEFORE or UNKNOWN_THROW_EXCEPTION
     * @throws UnsupportedOperationException if a comparison has already been performed and the comparator is locked
     * @throws IllegalArgumentException if the parameter <code>unknownObjectBehaviour</code> 
     * is not valid.
     */
    public void setUnknownObjectBehavior(int unknownObjectBehavior) {
        // TODO: complete
    }

    // Methods for adding items
    //-----------------------------------------------------------------------
    /** 
     * Adds an item, which compares as greater than all items known to the Comparator.
     * If the item is already known to the Comparator, its old position is
     * replaced with the new position.
     * 
     * @param obj  the {@link Object} to be added to the Comparator.
     * @return <code>true</code> if <code>obj</code> has been added for the first time, 
     * or <code>false</code> if it was already known to the Comparator.
     * @throws UnsupportedOperationException if a comparison has already been made
     * and the comparator is locked.
     */
    public boolean add(Object obj) {
        // TODO: complete
    	return false; // remove
    }

    /**
     * Adds a new item, which compares as equal to the given existing item.
     * 
     * @param existingObj  an item already in the Comparator's set of 
     *  known objects
     * @param newObj  an item to be added to the Comparator's set of
     *  known objects
     * @return true if <code>newObj</code> has been added for the first time, false if
     *  it was already known to the Comparator.
     * @throws IllegalArgumentException if <code>existingObject</code> is not in the 
     *  Comparator's set of known objects.
     * @throws UnsupportedOperationException if a comparison has already been made
     * and the comparator is locked.
     */
    public boolean addAsEqual(Object existingObj, Object newObj) {
        // TODO: complete
    	return false; // remove
    }

    // Comparator methods
    //-----------------------------------------------------------------------
    /** 
     * Compares two objects according to the order of this Comparator.
     * @param obj1  the first object to compare
     * @param obj2  the second object to compare
     * @return 
     * 
     * If both <code>obj1</code> and <code>obj2</code> are known to the Comparator,
     * the return value is similar to {@link Integer#compareTo(Integer)}:
     * <ul>
     * <li>Negative if <code>obj1</code> comes before <code>obj2</code> in the fixed order</li>
     * <li>Positive if <code>obj1</code> comes after <code>obj2</code> in the fixed order</li>
     * <li>Zero if <code>obj1</code> and <code>obj2</code> are equal in the fixed order</li></ul>
     * </ul>
     * For example, assuming the order {"Mercury"=0, "Venus"=1, "Earth"=1, "Mars"=2}:
     * <pre>
     * compare("Mercury","Earth")	-> -1	// "Mercury" is less than "Earth"
     * compare("Mars"   ,"Venus")	-> 1	// "Mars" is greater than "Venus"
     * compare("Venus"  ,"Earth")	-> 0	// "Venus" and "Earth" are equal
     * </pre>
     * <p>
     * Otherwise, if at least one object is unknown, the return value is negative, positive or 
     * zero depending on the unknown objects behaviour (set via {@link #setUnknownObjectBehavior(int)}).
     * <p>
     * For example, consider "Saturn", "Uranus" are objects unknown to this Comparator:
     * <ul>
     * <li>If the unknown objects behaviour is <code>UNKNOWN_BEFORE</code>, any unknown object comes before (is less than) any known one:</li>
     * <pre>
     * compare("Saturn","Mars"  )	-> -1 
     * compare("Mars"  ,"Saturn")	-> 1
     * compare("Uranus","Saturn")	-> 0 
     * </pre>
     * <li>If the unknown objects behaviour is <code>UNKNOWN_AFTER</code>, any unknown object comes after (is greater than) any known one:</li>
     * <pre>
     * compare("Saturn","Mars"  )	-> 1
     * compare("Mars"  ,"Saturn")	-> -1
     * compare("Uranus","Saturn")	-> 0 
     * </pre>
     * <li>If the unknown objects behaviour is <code>UNKNOWN_THROW_EXCEPTION</code> (the default), an exception is thrown indicating the unknown object:</li>
     * <pre>
     * compare("Saturn","Mars"  )	-> IllegalArgumentException("Saturn")
     * compare("Mars"  ,"Saturn")	-> IllegalArgumentException("Saturn")
     * compare("Uranus","Saturn")	-> IllegalArgumentException("Uranus")
     * </pre>
     * </ul>
     * <p>
     * @throws IllegalArgumentException if <code>obj1</code> or <code>obj2</code>
     * are not known to this Comparator and the unknown behaviour is <code>UNKNOWN_THROW_EXCEPTION</code>
     * (the unknown object is passed in in the exception string).
     * @throws UnsupportedOperationException if the unknown objects behavior has been set
     * to an invalid value (e.g., via {@link #setUnknownObjectBehavior(int)}).
     */
    public int compare(Object obj1, Object obj2) {
        // TODO: complete
    	return -1; // remove
    }

}
